from random import randint, seed, choice, shuffle
seed(47)

def save(fname, A, B, C, queries):
    assert all( 1 <= x <= A*B*C for x in queries )
    f = open(fname, 'w')
    print(A, B, C, file=f)
    print(len(queries), file=f)
    print(*queries, file=f)
    f.close()

def logrand(lo, hi):
    while True:
        b = randint(1, 60)
        x = randint(2**(b-1), (2**b)-1)
        if lo <= x <= hi: return x

def bigrand(hi):
    lo = max(1, int(0.8*hi + .5))
    return randint(lo, hi)

def gen_queries(q, lo, hi, f):
    queries = []
    if q >= 10: queries = [lo, lo+1, hi, hi-1]
    while len(queries) < q:
        x = f(lo, hi)
        if randint(0,1):
            d = x - lo
            x = hi - d
        queries.append(x)
    return queries

def gen_random(maxa, maxb, maxc, maxq):
    a, b, c, q = bigrand(maxa), bigrand(maxb), bigrand(maxc), bigrand(maxq)
    return (a,b,c,gen_queries(q,1,a*b*c,randint))

def gen_logrand(maxa, maxb, maxc, maxq):
    a, b, c, q = bigrand(maxa), bigrand(maxb), bigrand(maxc), bigrand(maxq)
    return (a,b,c,gen_queries(q,1,a*b*c,logrand))

def gen_consec(maxa, maxb, maxc, maxq):
    a, b, c, q = bigrand(maxa), bigrand(maxb), bigrand(maxc), bigrand(maxq)
    lo = logrand(1, a*b*c-q+2)
    values = list(range(lo,lo+q-1))
    values.append( choice(values) )
    shuffle(values)
    return (a,b,c,values)

save('00.sample.a.in', 3, 3, 3, [1,2,10] )
save('00.sample.b.in', 13, 2, 5, [70,130,24,108] )

save('01.a.in', *gen_random(20,20,20,20))
save('01.b.in', *gen_logrand(20,20,20,20))
save('01.c.in', *gen_consec(20,20,20,20))

save('02.a.in', *gen_random(180,180,180,1))
save('02.b.in', *gen_logrand(180,180,180,1))
save('02.c.in', *gen_logrand(180,180,180,1))

save('03.a.in', *gen_random(2500,2500,2500,1))
save('03.b.in', *gen_logrand(2500,2500,2500,1))
save('03.c.in', *gen_logrand(2500,2500,2500,1))

save('04.a.in', *gen_random(1,10**6,10**6,10**5))
save('04.b.in', *gen_logrand(1,10**6,10**6,10**5))
save('04.c.in', *gen_consec(1,10**6,10**6,10**5))

save('05.a.in', *gen_random(1,10**9,10**9,10**5))
save('05.b.in', *gen_logrand(1,10**9,10**9,10**5))
save('05.c.in', *gen_consec(1,10**9,10**9,10**5))

save('06.a.in', 999997, 999997, 999997, gen_queries(10**2, 1, 999997**3, randint) )
save('06.b.in', 999998, 999998, 999998, gen_queries(10**2, 1, 999998**3, logrand) )

save('07.a.in', 999997, 999997, 999997, gen_queries(10**5, 1, 999997**3, randint) )
save('07.b.in', 999998, 999998, 999998, gen_queries(10**5, 1, 999998**3, logrand) )

save('08.a.in', *gen_random(10**6,10**6,10**6,1))
save('08.b.in', *gen_logrand(10**6,10**6,10**6,1))
save('08.c.in', *gen_random(10**6,10**6,10**6,1))
save('08.d.in', *gen_logrand(10**6,10**6,10**6,1))

save('09.a.in', *gen_random(10**6,10**6,10**6,10**5))
save('09.b.in', *gen_logrand(10**6,10**6,10**6,10**5))
save('09.c.in', *gen_consec(10**6,10**6,10**6,10**5))

save('10.a.in', *gen_random(10**18,1,1,10**5))
save('10.b.in', *gen_random(10**9,10**9,1,10**5))
save('10.c.in', *gen_random(10**9,1,10**9,10**5))
save('10.d.in', *gen_random(1,10**9,10**9,10**5))
save('10.e.in', *gen_random(10**5,10**6,10**7,10**5))
save('10.f.in', *gen_logrand(10**7,10**5,10**6,10**5))
save('10.g.in', *gen_random(10**9,10**8,7,10**5))
save('10.h.in', *gen_random(10**8,7,10**9,10**5))
save('10.i.in', *gen_random(7,10**9,10**8,10**5))
save('10.j.in', *gen_consec(7,10**9,10**8,10**5))

# číslo sady      &  1 &   2 &    3 &      4 &      5 &      6 &      7 &      8 &      9 & 10 \\\hline\vphantom{\Large X}%
# $\max a$        & 20 & 180 & 2500 &      1 &      1 & $10^6$ & $10^6$ & $10^6$ & $10^6$ & \\
# $\max b,\max c$ & 20 & 180 & 2500 & $10^6$ & $10^9$ &        &        & $10^6$ & $10^6$ & \\
# $\max q$        & 20 &   1 &    1 &        &        & $10^2$ &        &      1 &        & \\
# iné             &    &     &      &        &        &  kocka &  kocka &        &        & \\

